from pathlib import Path
import subprocess
import os
import shutil
from ..core import IBenchmark, BenchParam, BenchmarkConfig
from ..util import get_proxy_env_vars


class HiBench(IBenchmark):
    """Hibridon benchmark
    """

    def __init__(self):
        bench_params = []
        bench_params.append(BenchParam('num_cores', BenchParam.Type.PARAM_TYPE_INT, 'the number of cores to use by this benchmark'))
        bench_params.append(BenchParam('hibridon_version', BenchParam.Type.PARAM_TYPE_STRING, 'the version of hibridon, in the form of a commit id'))
        bench_params.append(BenchParam('compiler_id', BenchParam.Type.PARAM_TYPE_STRING, 'the id of the compiler used in the benchmark'))
        # bench_params.append(BenchParam('compiler_version', BenchParam.Type.PARAM_TYPE_STRING, 'the version of the used compiler'))
        # bench_params.append(BenchParam('blas_id', BenchParam.Type.PARAM_TYPE_STRING, 'the id of the blas library used in the benchmark'))
        # bench_params.append(BenchParam('blas_version', BenchParam.Type.PARAM_TYPE_STRING, 'the version of the blas library used in the benchmark'))
        bench_params.append(BenchParam('test_id', BenchParam.Type.PARAM_TYPE_STRING, 'the name of the test to run (eg arch4_quick (about 2s on a core i5 8th generation) or nh3h2_qma_long (about 10min on a core i5 8th generation))'))
        bench_params.append(BenchParam('cmake_path', BenchParam.Type.PARAM_TYPE_STRING, 'the location of the cmake executable to use (eg "/opt/cmake/cmake-3.23.0/bin/cmake", or simply "cmake" for the one in the path)'))

        super().__init__(bench_id='hibench', bench_params=bench_params)

    def get_ram_requirements(self, config: BenchmarkConfig) -> int:
        GIBIBYTE_TO_BYTE = 1024 * 1024 * 1024
        ram_per_core = 0  # in bytes
        benchmark_test = config['test_id']
        if benchmark_test == 'arch4_quick':
            ram_per_core = int(1 * GIBIBYTE_TO_BYTE)
        elif benchmark_test == 'nh3h2_qma_long':
            ram_per_core = int(2.8 * GIBIBYTE_TO_BYTE)  # this was enough on physix48, but maybe we can reduce more
        else:
            assert f'unhandled benchmark_test : {benchmark_test}'
        return ram_per_core

    def execute(self, config: BenchmarkConfig, benchmark_output_dir: Path):

        git_repos_url = 'https://github.com/hibridon/hibridon'
        git_user = 'g-raffy'  # os.environ['HIBRIDON_REPOS_USER']
        git_pass_file = f'{os.getenv("HOME")}/.github/personal_access_tokens/bench.hibridon.cluster.ipr.univ-rennes1.fr.pat'
        hibridon_version = config['hibridon_version']
        test_id = config['test_id']  # eg arch4_quick or nh3h2_qma_long
        compiler_id = config['compiler_id']
        cmake_path = config['cmake_path']
        num_cores = config['num_cores']

        if benchmark_output_dir.exists():
            shutil.rmtree(benchmark_output_dir)
        benchmark_output_dir.mkdir(parents=True)

        src_dir = benchmark_output_dir / 'hibridon.git'
        output_dir = benchmark_output_dir / 'output'

        password_provider = f'{{"type": "password-file", "password-file-path": "{git_pass_file}"}}'
        source_tree_provider = f'{{"type": "git-cloner", "repos-url": "{git_repos_url}", "src-dir": "{src_dir}", "code-version": "{hibridon_version}", "git-user": "{git_user}", "password-provider": {password_provider}}}'
        benchmark_command = f'ctest --output-on-failure -L ^{test_id}$'

        cmake_options = [
            '-DCMAKE_BUILD_TYPE=Release',  # build in release mode for highest performance
            '-DBUILD_TESTING=ON'  # enable hibridon tests
        ]

        env_vars_bash_commands = ''
        if compiler_id == 'ifort':
            env_vars_bash_commands = 'module load compilers/ifort/latest'
            cmake_options.append('-DCMAKE_Fortran_COMPILER=ifort')  # use intel fortran compiler
            cmake_options.append('-DBLA_VENDOR=Intel10_64lp')  # use 64 bits intel mkl with multithreading
        elif compiler_id == 'gfortran':
            env_vars_bash_commands = ''
            cmake_options.append('-DCMAKE_Fortran_COMPILER=gfortran')  # use gfortran compiler
        else:
            assert f'unhandled compiler_id : {compiler_id}'

        shell_command = ''
        if len(env_vars_bash_commands) > 0:
            shell_command += f'{env_vars_bash_commands} && '
        shell_command += f'{get_proxy_env_vars()} starbench --source-tree-provider \'{source_tree_provider}\' --num-cores {num_cores} --output-dir={output_dir} --cmake-path={cmake_path} {" ".join([f"--cmake-option={option}" for option in cmake_options])} --benchmark-command=\'{benchmark_command}\''
        subprocess.run(shell_command, shell=True, check=True, executable='/bin/bash')
