from typing import List, Dict, Union
from enum import Enum
import abc
from pathlib import Path

BenchmarkId = str  # a unique name for a benchmark, eg 'matmul1'
BenchParamId = str
BenchParamType = Union[int, str]
BenchmarkConfig = Dict[BenchParamId, BenchParamType]


class BenchParam():
    '''a parameter of a benchmark

    for example the id of the compiler, the cpu id, the size of the matrix, etc.

    '''
    class Type(Enum):
        PARAM_TYPE_STRING = 0
        PARAM_TYPE_INT = 1

    name: BenchParamId  # the name of the parameter, eg 'matrix_size'
    param_type: Type  # the type of the parameter, eg 'PARAM_TYPE_INT'
    description: str  # the description of the parameter, eg 'the size n of the n*n matrix '

    def __init__(self, name: str, param_type: Type, description: str):
        self.name = name
        self.param_type = param_type
        self.description = description


class IBenchmark(abc.ABC):

    bench_id: BenchmarkId  # a unique name for this benchmark, eg 'matmul1'
    bench_params: List[BenchParam]

    def __init__(self, bench_id: str, bench_params: List[BenchParam]):
        self.bench_id = bench_id
        self.bench_params = bench_params

    @abc.abstractmethod
    def get_ram_requirements(self, config: BenchmarkConfig) -> int:
        """returns the ram requirements for this benchmark, in bytes
        """

    @abc.abstractmethod
    def execute(self, config: BenchmarkConfig, benchmark_output_dir: Path):
        """execute the benchmark for the given config
        """

    def validate_config(self, config: BenchmarkConfig):
        """checks that all benchmark parameters have been set in the given config"""
        for bench_param in self.bench_params:
            try:
                _ = config[bench_param.name]
            except KeyError:
                assert False, f'failed to find the benchmark parameter {bench_param.name} in the benchmark config'
        # check that all parameters in benchmark config exist as parameters for this benchmark
        for param_name in config.keys():
            param_exists = False
            for bench_param in self.bench_params:
                if bench_param.name == param_name:
                    param_exists = True
                    break
            assert param_exists, f'parameter {param_name} doesn\'t exist for benchmark {self.bench_id}'
